##########################################################################
## Copyright(c) 2006-2018 ThreatSTOP, Inc. All Rights Reserved
##
## Package Maintainer : support@threatstop.com
##
## NOTICE: All information contained herein is, and remains the property
## of ThreatSTOP, Inc. and its suppliers, if any. The intellectual and
## technical concepts contained herein are proprietary to ThreatSTOP, Inc.
## and its suppliers and certain aspects thereof are protected by United
## States Patent No. 8,533,822 and United States Patent No. 8,869,237,
## and are protected by trade secret or copyright law.
##
## US Government Entities: The ThreatSTOP service, software and
## documentation, as applicable, are "commercial computer software" and
## "commercial computer software documentation" developed exclusively at
## private expense by Threatstop, Inc. ("Threatstop"). Pursuant to FAR
## 12.212 or DFARS 227 7202 and their successors, as applicable, use,
## reproduction and disclosure of the software and documentation is
## governed by the terms of Threatstop's commercial agreement.
#########################################################################
#
package ThreatStop::MultiDevice::Types::CheckPoint;

=head1 SYNOPSIS

 This module provides device specific command line and prompt processing functions.
 For Check Point devices
=cut

use strict;
use warnings;
use Getopt::Modular;
use Term::Prompt;
use IO::Interactive qw(is_interactive);
use Exception::Class;
use ThreatStop::MultiDevice::Admin;
use 5.010;    #enable perl 5.10 features.
use Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK =
  qw(device_parseCmdOpts device_doPrompts device_doAdvancedPrompts device_hasMandatoryCfgOpts verifyDeviceAccess device_ShowCfgData device_cmdAdd device_cmdConfigure device_cmdRemove device_configValidation device_promptForDeviceCredentials);

# define command line options using defaults from our device template file.
BEGIN {
    my $defaults = {};
    my $err_msg;
    loadConfFileData( "/opt/threatstop/templates/checkpoint/device.conf",
        $defaults, \$err_msg );
}

my $COLOR_DEFS = {
    'normal'  => '',
    'red'     => "\033[1;31m",
    'blue'    => "\033[1;34m",
    'cyan'    => "\033[1;36m",
    'green'   => "\033[0;32m",
    'okblue'  => "\033[94m",
    'okgreen' => "\033[92m",
};

my $STYLE_DEFS = {
    'normal'    => '',
    'reverse'   => "\033[;7m",
    'reset'     => "\033[0;0m",
    'bold'      => "\033[;1m",
    'underline' => "\033[4m",
    'header'    => "\033[95m",
    'fail'      => "\033[91m",
    'endc'      => "\033[0m",
    'warning'   => "\033[93m",
};

=head3 print_colorized
  simple method to print colorized text via control codes to terminal.
    args:
        text, string
        color, string (red, blue, green)
        style, string (normal*, reverse, reset, bold, underline, header, fail, endc, warning)
    returns:
        nothing
=cut

sub print_colorized {
    my ( $text, $color, $style ) = @_;
    $color ||= 'normal';
    $style ||= 'normal';

    print( $STYLE_DEFS->{$style} . $COLOR_DEFS->{$color} . $text . "\033[0m" );
}

# required interface routine
# see Admin.pm's use for explanation of what can be added
# returns :
#   nothing, but may adjust the passed checks reference
sub device_configValidation {
}

#
# pass in device configuration hashref and fill it in based on command line options
#
sub device_parseCmdOpts {
    my $cfg = shift;
    $cfg->{allow_list} =
      'dns.threatstop.local';    # This is here to as a default. it is unused.
     # but we need a default value. The skip happens as part of the default.conf see: skip_vars
}

#
# pass in device configuration hashref and fill it in based on prompts.
#
sub device_doPrompts {
    my $cfg = shift;
    $cfg->{blade} = prompt(
        's',
        'Check Point Blade Enabled, Anti-Bot or Anti-Virus:',
        qq|ab / av|,
        $cfg->{blade},
        sub { my $var = shift; return 1 if $var =~ m/a[v|b]/; return 0; }
    );

    $cfg->{maxpolicysize} =
      prompt( 'n', 'Max policy size:', '', $cfg->{maxpolicysize} );

    # p $cfg->{blade};
}

sub device_doAdvancedPrompts {
}

# return Text::Table data for tsadmin show subcommand
sub device_ShowCfgData {
    my $cfg = shift;

    my $ipaddr = `hostname -I`;

    # remove \n and trailing space
    $ipaddr =~ s/\s+$//;

    my $burl =
        "http://"
      . $ipaddr
      . ":8002/"
      . $cfg->{'device_id'}
      . "/threatstop-block.csv";

    # my $aurl =
    #     "http://"
    #   . $ipaddr
    #   . ":8002/"
    #   . $cfg->{'device_id'}
    #   . "/threatstop-allow.csv";
    print_colorized(
"\nSample IP Defense Check Point Custom Threat Intelligence Feed setup command:",
        '', 'underline'
    );
    print_colorized(
        "\nioc_feeds add --feed_name threatstop-block --transport http \\\n",
        'green' );
    print_colorized( "--resource '$burl'\n\n", 'green' );
    print_colorized( "Sample Check Point logging setup command:",
        '', 'underline' );
    print_colorized(
"\ncp_log_export add name tssyslog target-server $ipaddr target-port 514 protocol udp format syslog\n\n",
        'green'
    );

    # my @rows = ( [ 'Block List URL', $burl ], [ 'Allow List URL', $aurl ] );
    my @rows = ( [ 'Block List URL', $burl ] );
    return @rows;
}

#
# return
#    0 something is wrong
#    1 if all mandatory configuration options are set
#
sub device_hasMandatoryCfgOpts {
    my $cfg = shift;
    return 1;
}

#
# Validation subroutine used by tsadmin to tell it we can talk to the device OK.
#
# args :
#    configuration reference
#
# returns :
#    0 for failure
#    1 for success
#
sub verifyDeviceAccess {
    return 1;
}

#
# helper used by "add" and "configure" callback
#
# called with reference to config hash
#
# returns :
#   1 for success
#   0 for failure
sub _verifyConfiguredSettings {
    my $config = shift;
    return 1;
}

=head3 enable_nginx

 helper used by "add" to add nginx config

 returns :
   1 for success
   0 for failure
=cut

sub enable_nginx {
    my $nginx_available = '/etc/nginx/sites-available/checkpoint.conf';
    my $nginx_enabled   = '/etc/nginx/sites-enabled/checkpoint.conf';
    unless ( -f $nginx_available ) {
        printMsg( "ERROR", "nginx conf not installed. check $nginx_available" );
        return 0;
    }

    if ( -l $nginx_enabled ) {
        printMsg( "DEBUG", "Symlink exists - nginx already enabled" );
        return 1;
    }
    else {
        printMsg( "DEBUG", "Symlinking nginx config" );
        my $cmd = "sudo ln -s -f $nginx_available $nginx_enabled";
        system($cmd ) == 0 or die "System call failed: $cmd $!";

# according to https://www.cyberciti.biz/faq/nginx-linux-restart/ should work for CENTOS/RHEL/Ubuntu
        my $nginx_reload_cmd = 'sudo service nginx reload';
        printMsg( "DEBUG", "restarting nginx" );
        system($nginx_reload_cmd ) == 0
          or die "System call failed: $nginx_reload_cmd $!";

        # check if we have a listen port 8002
        sleep(2);    # wait for nginx to finish restarting.
        my $netstat_port = `netstat -an|grep ':8002 '|grep LISTEN`;
        if ($netstat_port) {
            printMsg( "DEBUG",
                "We have port 8002 listening \n\t$netstat_port" );
        }
        else {
            printMsg( "DEBUG",
                "We not have anything listening on port 8002 \n\t$netstat_port"
            );
            printMsg( "ERROR", "failed starting nginx" );
            return 0;
        }
        return 1;
    }
    return 0;
}

=head3 enable_nginx

 helper used by "remove" to remove nginx config when no other checkpoint device are left.

 returns :
   1 for success
   0 for failure
=cut

sub disable_nginx {
    my $nginx_available = '/etc/nginx/sites-available/checkpoint.conf';
    my $nginx_enabled   = '/etc/nginx/sites-enabled/checkpoint.conf';
    if ( -l $nginx_enabled ) {
        printMsg( "DEBUG", "removing nginx symlink" );
        my $cmd = "sudo rm " . $nginx_enabled;
        system($cmd ) == 0 or die "System call failed: $cmd $!";

        printMsg( "DEBUG", "restarting nginx" );
        my $nginx_reload_cmd = 'sudo service nginx reload';
        system($nginx_reload_cmd ) == 0
          or die "System call failed: $nginx_reload_cmd $!";
    }
    else {
        printMsg( "DEBUG", "nginx not symlinked skipping." );
    }

    sleep(2);    # wait for nginx to finish restarting.
    my $netstat_port = `netstat -an|grep 8002|grep LISTEN`;
    if ($netstat_port) {
        printMsg( "WARN",
            "We still have port 8002 listening \n\t$netstat_port" );
        return 0;
    }
    else {
        printMsg( "DEBUG",
            "We don't have anything listening on port 8002 \n\t$netstat_port" );
    }
    return 1;
}

sub enable_nginx_acl {
    my $cfg = shift;
    # Apply ACL

    if (! -e "/etc/nginx/sites-available/checkpoint-acl.d") {
        printMsg( "DEBUG", "Adding NGINX acl.d dir" );
        system("sudo /bin/mkdir -m 0775 /etc/nginx/sites-available/checkpoint-acl.d");
        system("sudo /bin/chgrp threatstop /etc/nginx/sites-available/checkpoint-acl.d");
    }

    my $device_id = $cfg->{device_id};
    my $device_ip = $cfg->{device};
    printMsg( "DEBUG", "Adding NGINX $device_id ACL" );
    open(my $fh, ">", "/etc/nginx/sites-available/checkpoint-acl.d/$device_id.conf");
    print $fh "allow $device_ip;\n";
    close $fh;
    system("sudo /bin/chown threatstop:threatstop /etc/nginx/sites-available/checkpoint-acl.d/$device_id.conf");

    # Entry for TSCM (troubleshooting)

    my $tscm_acl = '/etc/nginx/sites-available/checkpoint-acl.d/tscm.conf';

    unless ( -f $tscm_acl ) {
        printMsg( "DEBUG", "Adding NGINX TSCM ACL" );
        my $tscmip = `hostname -I`;
        $tscmip =~ s/^\s+|\s+$//g;
        $tscmip =~ s/\n//g;
        open(my $fh2, ">", $tscm_acl);
        print $fh2 "allow $tscmip;\n";
        close $fh2;
        system("sudo /bin/chown threatstop:threatstop $tscm_acl");
    }
    printMsg( "DEBUG", "NGINX ACL Successfully added" );
}

sub disable_nginx_acl {
    my $cfg = shift;
    # Apply ACL

    my $device_id = $cfg->{device_id};
    my $acl_file = "/etc/nginx/sites-available/checkpoint-acl.d/$device_id.conf";
    system("sudo rm $acl_file");
    if (-f $acl_file) {
        printMsg( "DEBUG", "NGINX ACL Failed removing ($acl_file)" );
    } else {
        printMsg( "DEBUG", "NGINX ACL Successfully removed ($acl_file)" );
    }

}

=head3 device_cmdAdd

 required callbacks (refer to GS-446)

 args :
   configuration hash reference

 returns :
   1 for success
   0 for failure
=cut

sub device_cmdAdd {

    my $config = shift;
    printMsg( "DEBUG", "In device callback for cmdAdd()" );

    if ( enable_nginx() ) {
        printMsg( "DEBUG", "device nginx conf enabled" );
        &enable_nginx_acl($config);
    }
    device_ShowCfgData($config);

    return _verifyConfiguredSettings($config);
}

=head3 device_cmdRemove

 this satisfies the mandatory interface with TSCM
 returns

 0 failure, and the TSCM should stop the removal
 1 all good, TSCM may continue
=cut

sub device_cmdRemove {
    my $config = shift;

    # Remove lists
    my $device_base = '/usr/share/checkpoint_root/';
    my $device_root = $device_base . $config->{'device_id'};
    system("/bin/rm -rf $device_root");
    my $devices_left = `ls $device_base|wc -l`;

    &disable_nginx_acl($config); # remove per-device ACL

    if ( $devices_left == 0 ) {

        # we have no more devices remove nginx config
        if ( disable_nginx() ) {
            printMsg( "DEBUG", "device nginx conf disabled" );
        }

        my $tscm_acl = '/etc/nginx/sites-available/checkpoint-acl.d/tscm.conf';
        if ( -f $tscm_acl ) {
            system("sudo /bin/rm $tscm_acl");
        }
    }
    return 1;
}


sub device_cmdConfigure {

    my $config = shift;
    printMsg( "DEBUG", "In device callback for cmdConfigure()" );

    return _verifyConfiguredSettings($config);
}

# pass in device configuration hashref and fill it in based on prompts.
# returns nothing interesting
#
# this satisfies the TSCM core interface
sub device_promptForDeviceCredentials {
    my $cfg = shift;

    # CheckPoint has no additional credentials to prompt for
}

1;
